# -*- coding: utf-8 -*-

import os

__title__ = "SWAT-EM - Specific Winding Analyse Tool for electrical machines"
__version__ = "0.6.0"
__author__ = "Martin Baun"
__license__ = "MIT License"
__copyright__ = "Copyright 2018-2019 Martin Baun"

from swat_em.datamodel import datamodel, project
